<?php

if (!defined('ABSPATH')) {

	die('This file cannot be accessed directly');
}

if (!function_exists('init_gardeshpay_gateway_pv_class')) {

	add_action('plugins_loaded', 'init_gardeshpay_gateway_pv_class');

	function init_gardeshpay_gateway_pv_class()
	{
		add_filter('pro_vip_currencies_list', 'currencies_check');

		function currencies_check($list)
		{
			if (!in_array('IRT', $list)) {

				$list['IRT'] = array(

					'name'   => 'تومان ایران',
					'symbol' => 'تومان',
				);
			}

			if (!in_array('IRR', $list)) {

				$list['IRR'] = array(

					'name'   => 'ریال ایران',
					'symbol' => 'ریال',
				);
			}

			return $list;
		}

		if (class_exists('Pro_VIP_Payment_Gateway') && !class_exists('Pro_VIP_gardeshpay_Gateway')) {

			class Pro_VIP_gardeshpay_Gateway extends Pro_VIP_Payment_Gateway
			{
				public

					$id            = 'gardeshpay',
					$settings      = array(),
					$frontendLabel = 'درگاه پرداخت و کیف پول الکترونیک gardeshpay.ir',
					$adminLabel    = 'درگاه پرداخت و کیف پول الکترونیک gardeshpay.ir';

				public function __construct()
				{
					parent::__construct();
				}

				public function beforePayment(Pro_VIP_Payment $payment)
				{
					if (extension_loaded('curl')) {

						$api_key  = $this->settings['api_key'];
						$order_id = $payment->paymentId;
						$callback = add_query_arg('order', $order_id, $this->getReturnUrl());
						$amount   = intval($payment->price);

						if (pvGetOption('currency') == 'IRT') {

							$amount = $amount * 10;
						}

						$params = array(
							'token'          => preg_replace('/\s+/', '', $api_key),
							'amount'       => $amount,
							'callback'     => urlencode($callback),
							'invoiceNumber' => $order_id,
							'invoiceDate' => date('Y-m-d')
						);

						$result = $this->common('https://ipg.gardeshpay.ir/v1/provider/payment/getToken', $params);

						if ($result && $result->seccess) {

							$payment->key = $order_id;

							$payment->user = get_current_user_id();
							$payment->save();

							$message     = 'شماره تراکنش ' . $result->token;
							$gateway_url = $result->data->url . '/' . $result->data->token;

							pvAddNotice($message, 'error');

							wp_redirect($gateway_url);
							exit;

						} else {

							$message = 'در ارتباط با وب سرویس gardeshpay.ir خطایی رخ داده است';
							$message = isset($result->code) ? $result->code : $message;

							pvAddNotice($message, 'error');

							$payment->status = 'trash';
							$payment->save();

							wp_die($message);
							exit;
						}

					} else {

						$message = 'تابع cURL در سرور فعال نمی باشد';

						pvAddNotice($message, 'error');

						$payment->status = 'trash';
						$payment->save();

						wp_die($message);
						exit;
					}
                }

				public function afterPayment()
				{
					if (isset($_GET['order'])) {

						$order_id = sanitize_text_field($_GET['order']);

					} else {

						$order_id = NULL;
					}

					if ($order_id) {

                        $payment = new Pro_VIP_Payment($order_id);

						if (isset($_GET['status']) && isset($_GET['token']) ) {

							$status        = sanitize_text_field($_GET['status']);
							$trackingNumber         = sanitize_text_field($_GET['trackingNumber']);

							if (isset($status) && $status == 1) {

								$api_key = $this->settings['api_key'];

								$params = array (

									'token'     => preg_replace('/\s+/', '', $api_key),
									'trackingNumber' => $trackingNumber
								);

								$result = $this->common('https://ipg.gardeshpay.ir/v1/provider/payment/verify', $params);

								if ($result->success) {

									$card_number = isset($result->cardNumber) ? sanitize_text_field($result->cardNumber) : 'Null';
                                    $invoiceNumber = isset($result->invoiceNumber) ? sanitize_text_field($result->invoiceNumber) : 'Null';

									$amount  = intval($payment->price);

									if (pvGetOption('currency') == 'IRT') {

										$amount = $amount * 10;
									}

									if ($amount == $result->amount) {

										$message = 'تراکنش شماره ' . $result->transId . ' با موفقیت انجام شد';

										pvAddNotice($message, 'success');

										$payment->status = 'publish';
										$payment->save();

										$this->paymentComplete($payment);

									} else {

										$message = 'رقم تراكنش با رقم پرداخت شده مطابقت ندارد';

										pvAddNotice($message, 'error');

										$payment->status = 'trash';
										$payment->save();

										$this->paymentFailed($payment);
									}

								} else {

									$message = 'در ارتباط با وب سرویس gardeshpay.ir و بررسی تراکنش خطایی رخ داده است';
									$message = isset($result->errorMessage) ? $result->errorMessage : $message;

									pvAddNotice($message, 'error');

									$payment->status = 'trash';
									$payment->save();

									$this->paymentFailed($payment);
								}

							} else {

								$message = $message ? $message : 'تراكنش با خطا مواجه شد و یا توسط پرداخت کننده کنسل شده است';

								pvAddNotice($message, 'error');

								$payment->status = 'trash';
								$payment->save();

								$this->paymentFailed($payment);
							}

						} else {

							$message = 'اطلاعات ارسال شده مربوط به تایید تراکنش ناقص و یا غیر معتبر است';

							pvAddNotice($message, 'error');

							$payment->status = 'trash';
							$payment->save();

							$this->paymentFailed($payment);
						}

					} else {

						$message = 'شماره سفارش ارسال شده غیر معتبر است';

						pvAddNotice($message, 'error');
					}
				}

				public function adminSettings(PV_Framework_Form_Builder $form)
				{
					$form->textfield('api_key')->label('کلید API');
				}

				private static function common($url, $params)
				{
					$ch = curl_init();

					curl_setopt($ch, CURLOPT_URL, $url);
					// curl_setopt($ch, CURLOPT_POST, TRUE);
					curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
					curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
					curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, FALSE);
					curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($params));

					$response = curl_exec($ch);
					$error    = curl_errno($ch);

					curl_close($ch);

					$output = $error ? FALSE : json_decode($response);

					return $output;
				}
			}

			Pro_VIP_Payment_Gateway::registerGateway('Pro_VIP_gardeshpay_Gateway');
		}
	}
}
